#include "camera.h"
#include "matrix4.h"
#include "object3d.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include <cmath>
#include <sstream>

Camera::Camera (Vector3D e, Vector3D p, Vector3D u)
{
        Eye=e;
        Point=p;
        Up=u;
}

Camera::Camera (Vector3D e, Vector3D p)
{
        Eye=e;
        Point=p;
        Up=Vector3D(0,1.0,0);
}

Camera::Camera ()
{
        Eye=Vector3D(0,0,0);
        Point=Vector3D(0,0,-1);
        Up=Vector3D(0,1.0,0);
}

Camera::~Camera()
{
}

void Camera::SetGLCamera ()
{
        CalcMatrix();
        glMatrixMode (GL_MODELVIEW);
        glLoadIdentity ();
        Matrix.glMultMatrix();
        glTranslated(-Eye.x,-Eye.y,-Eye.z);
}

void Camera::CalcMatrix()
{
        Matrix.CreateCameraMatrix(Eye,Point,Up);

}

void Camera::SetEye(Vector3D e)
{
        Eye=e;
}

void Camera::SetPoint(Vector3D p)
{
        Point=p;
}

void Camera::SetUp(Vector3D u)
{
        Up=u;
}

Vector3D Camera::GetEye()
{
        return Eye;
}

string Camera::GetEyeString()
{
        ostringstream oss;
        oss.str("");
        oss<<"camera: "<< Eye.GetString();
        return oss.str();
}
