#ifndef _COPTER_H

#define _COPTER_H 1

#include "mobile.h"
#include <SDL.h>
#include "steer.h"
#include "text2d.h"

class Copter:public Mobile
                        /**< Controlable helicopter. */
{
        private:
                Steer *CollectiveSteer; /**< steer which controls Y axis */
                Steer *RearRotorSteer;  /**< steer for yaw rotation */
                Steer *LeftRightSteer;  /**< steer for roll rotation */
                Steer *ForwardBackSteer; /**< steer for pitch rotation */

                double RollMax; /**< max roll value */
                double PitchMax; /**< max pitch value */

                double RollSpeed; /**< speed of changing roll */
                double PitchSpeed; /**< speed of changing pitch */
                double YawSpeed; /**< speed of changing yaw */
                double CollectiveSpeed; /**< speed of changing height */
                double LeftRightYawSpeed; /**< how much roll causes changing
                			       yaw */

                double LeftRightSpeed; /**< how much roll causes copter moves
                                            left/right */
                double ForwardBackSpeed; /**< how much pitch causes copter
                			      moves forward/back */

                double MainRotorSpeed; /**< how fast main rotor rotates */
                double MainRotorPhase; /**< phase of main rotor */
                double RearRotorSpeed; /**< how fast rear rotor rotates */
                double RearRotorPhase; /**< phase of reat rotor */

                Vector3D Friction; /**< air friction */

                double CentrifugalRoll; /**< how fast roll resumes to 0 */
                double CentrifugalPitch; /**< how fast pitch resumes to 0 */

                void SteerStep(Uint8 *keys, int k1, int k2, Steer *s, double t);
                /**< changes steer with keys */

                void RotorsStep(double t);
                /**< updates rotors */

                void CalcAcceleration();
                /**< calculates acceleration from forces */

                void StepYaw(double t);
                /**< changes yaw angle */

                void DrawInfoColumn(double x, double y, double w, double h, int steps, double value);
                /**< draws vertical bar with information about one value */

        public:
                Copter(MetaMesh *meshes, Vector3D _position, Landscape *l);
                /**< meshes is set of meshes for drawing copter, _position is
                 * initial position of copter, l is pointer to landscape, for
                 * collision testing */

                void DrawInfo(Text2D *text2d);
                /**< draws information about helicopter state on screen */

                void Step(double t, Uint8 *keys);
                /**< step of action, t is time (length) of frame, keys is array
                 * of pressed keys */
};

#endif
