#include "gamemanager.h"
#include <iostream>
#include <SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <ctime>
#include <cstdlib>
#include <sstream>
#include "common.h"

GameManager::GameManager(int _ScreenX, int _ScreenY, int _ScreenBPP)
{
        GLfloat fog_color[] = { 0.3, 0.55, 0.85, 1.0 };

        SDL_Surface *screen;
        cout <<"new GameManager\n";
        SDL_Init(SDL_INIT_VIDEO);
        atexit(SDL_Quit);
        SDL_ShowCursor(SDL_DISABLE);
        TexturesDirectory=GetTexturesDirectory();
        LocalDataDirectory=GetLocalDataDirectory();
        ScreenX=_ScreenX;
        ScreenY=_ScreenY;
        screen = SDL_SetVideoMode (ScreenX, ScreenY, _ScreenBPP, SDL_OPENGL);
        if (screen==NULL)
        {
                cout <<"GameManager error - SDL_SetVideoMode\n";
                exit(-1);
        }
        glViewport (0, 0, ScreenX, ScreenY);
        glFogfv (GL_FOG_COLOR, fog_color);
        glClearColor(fog_color[0],fog_color[1],fog_color[2],fog_color[3]);
        //    glFogf (GL_FOG_DENSITY, 0.0000002f);
        glFogi(GL_FOG_MODE,GL_LINEAR);
        //    glHint(GL_FOG_HINT, GL_DONT_CARE);

        srand(time(NULL));
        camera=new Camera();
        cameraView=0;
        text2d=new Text2D(TexturesDirectory+"font.png",0.018);
        fpscounter=new FpsCounter(text2d);
        fpscounter->Reset();
        landscape=NULL;
        LandscapePrimitive=GL_TRIANGLES;
        //   LandscapePrimitive=GL_LINES;
        done=0;
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        Set2DMode();
        text2d->Print(0.45,0.5,"Please wait...");
        SDL_GL_SwapBuffers();
}

GameManager::~GameManager()
{
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        Set2DMode();
        text2d->Print(0.45,0.5,"Bye bye!");
        SDL_GL_SwapBuffers();
        delete camera;
        delete text2d;
        delete fpscounter;
        if (landscape)
                delete landscape;
}

void GameManager::Set3DMode(float ViewAngle, float Near, float Far)
{
        glMatrixMode (GL_PROJECTION);
        glLoadIdentity ();
        gluPerspective(ViewAngle,(float)ScreenX/ScreenY,Near,Far);
        glMatrixMode (GL_MODELVIEW);
        ViewDistance=Far;
        glEnable(GL_FOG);
        glFogf(GL_FOG_START, Far/6);
        glFogf(GL_FOG_END, Far);
}

void GameManager::Set2DMode()
{
        glMatrixMode (GL_PROJECTION);
        glLoadIdentity ();
        glOrtho(0.0,1.0,0.0,1.0,0.0,1.0);
        glMatrixMode (GL_MODELVIEW);
        glLoadIdentity ();
        glDisable(GL_FOG);
}

void GameManager::GLCheck()
{
        GLint texMaxUnit;
        GLint texsize;
        GLint depth_bits;
        const GLubyte *s;

        glGetIntegerv( GL_MAX_TEXTURE_UNITS_ARB, &texMaxUnit );
        cout <<"texMaxUnit="<<texMaxUnit<<"\n";
        glGetIntegerv(GL_MAX_TEXTURE_SIZE, &texsize);
        cout <<"texsize="<<texsize<<"\n";
        glGetIntegerv(GL_DEPTH_BITS, &depth_bits);
        cout <<"depth_bits="<<depth_bits<<"\n";
        s=glGetString(GL_RENDERER);
        cout <<"renderer: "<<s<<"\n";
}

void GameManager::StartFrame()
{
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        SDL_PumpEvents();
}

void GameManager::EndFrame()
{
        SDL_GL_SwapBuffers();
}

void GameManager::FollowWithCamera()
{
        switch (cameraView)
        {
        case 1:
                DistantCameraView();
                break;
        case 2:
                EternalCameraView();
                break;
        case 3:
                SideCameraView();
                break;
        case 0:
        default:
                FrontCameraView();
                break;
        }
}

void GameManager::FrontCameraView()
{
        Vector3D cameraEye = Vector3D(0,3.2,-4.5);
        cameraEye = copter->GetOrientation()*cameraEye;
        cameraEye += copter->GetPosition();
        camera->SetEye(cameraEye);

        Vector3D cameraPoint = Vector3D(0,2.0,-40.0);
        cameraPoint = copter->GetOrientation()*cameraPoint;
        cameraPoint = copter->GetPosition()+cameraPoint;
        camera->SetPoint(cameraPoint);

        Vector3D setup = Vector3D(0,1,0);
        setup = copter->GetOrientation()*setup;
        camera->SetUp(setup);
}

void GameManager::SideCameraView()
{
        Vector3D cameraEye = Vector3D(-80,3,0);
        cameraEye = copter->GetOrientation()*cameraEye;
        cameraEye.y = 0;
        cameraEye += copter->GetPosition();
        camera->SetEye(cameraEye);

        camera->SetPoint(copter->GetPosition());

        camera->SetUp(Vector3D(0,1,0));
}

void GameManager::DistantCameraView()
{
        Vector3D cameraEye = Vector3D(0,3,80.0);
        cameraEye = copter->GetOrientation()*cameraEye;
        cameraEye.y = 0;
        cameraEye += copter->GetPosition();
        camera->SetEye(cameraEye);

        camera->SetPoint(copter->GetPosition());

        camera->SetUp(Vector3D(0,1,0));
}

void GameManager::EternalCameraView()
{
        Vector3D cameraEye = Vector3D(0,25,-30);
        cameraEye = copter->GetOrientation()*cameraEye;
        cameraEye += copter->GetPosition();
        camera->SetEye(cameraEye);

        Vector3D cameraPoint = Vector3D(0,-40,40);
        cameraPoint = copter->GetOrientation()*cameraPoint;
        cameraPoint = copter->GetPosition()+cameraPoint;
        camera->SetPoint(cameraPoint);

        Vector3D setup = Vector3D(0,1,0);
        setup = copter->GetOrientation()*setup;
        camera->SetUp(setup);
}

void GameManager::SetGLCamera()
{
        camera->SetGLCamera();
}

void GameManager::LoadLandscape(string filename, double _vd, int MaxTriangles, int PatchLevel)
{
        landscape=new Landscape(filename,PatchLevel,_vd,LandscapeTextureSize, MaxTriangles);
        landscape->CreatePatches(LocalDataDirectory);
}

void GameManager::DrawLandscape(double Quality)
{
        Vector3D Eye;
        Eye=camera->GetEye();
        landscape->Update(&Eye,Quality);
        landscape->Draw(Eye,LandscapePrimitive);
}

void GameManager::Print(double x, double y,string s)
{
        text2d->Print(x,y,s);
}

int GameManager::GetPatchesX()
{
        return landscape->GetPatchesX();
}

int GameManager::GetPatchesZ()
{
        return landscape->GetPatchesZ();
}

void GameManager::PrintCameraEye()
{
        Print(0.05,0.95,camera->GetEyeString());
}

void GameManager::PrintQueuesSize()
{
        if (landscape)
                Print(0.05,0.90,landscape->GetQueuesSizeString());
}

void GameManager::InitCopter(string filename)
{
        MetaMeshes["copter"]=MetaMesh(filename);
        MetaMeshes["copter"].GenerateLists();
        copter=new Copter(&(MetaMeshes["copter"]),Vector3D(5000,1200,-15000),landscape);
        copter->SetTextures("maps/default.copter");
}

void GameManager::DrawCopter()
{
        copter->Draw();
}

void GameManager::CopterStep()
{
        static int last_t;
        int t,delta_t;
        t=SDL_GetTicks();
        delta_t=t-last_t;
        last_t=t;
        Uint8 *keys = SDL_GetKeyState(NULL);
        copter->Step(delta_t/1000.0,keys);
        if (copter->Crashed())
                done=1;
}

void GameManager::UpdateFpsCounter()
{
        fpscounter->AddFrame();
        fpscounter->Display();
}

void GameManager::SwitchCameraView()
{
        Uint8 *keys = SDL_GetKeyState(NULL);
        if (keys[SDLK_F1]==SDL_PRESSED) cameraView = 0;
        if (keys[SDLK_F2]==SDL_PRESSED) cameraView = 1;
        if (keys[SDLK_F3]==SDL_PRESSED) cameraView = 2;
        if (keys[SDLK_F4]==SDL_PRESSED) cameraView = 3;
}

void GameManager::SetLight()
{
        GLfloat light_position[4];
        GLfloat light_diffuse[4];
        GLfloat light_specular[4];
        Vector3D LightDirection;
        LightDirection=landscape->GetLightDirection();

        light_position[0]=LightDirection.x;
        light_position[1]=LightDirection.y;
        light_position[2]=LightDirection.z;
        light_position[3]=0.0;
        glLightfv(GL_LIGHT0,GL_POSITION,light_position);

        light_diffuse[0]=3.0;
        light_diffuse[1]=3.0;
        light_diffuse[2]=3.0;
        light_diffuse[3]=1.0;
        glLightfv(GL_LIGHT0,GL_DIFFUSE,light_diffuse);

        light_specular[0]=3.0;
        light_specular[1]=3.0;
        light_specular[2]=3.0;
        light_specular[3]=1.0;
        glLightfv(GL_LIGHT0,GL_SPECULAR,light_specular);

}

void GameManager::KeysStep()
{
        Uint8 *keys;
        keys=SDL_GetKeyState(NULL);
        if (keys[SDLK_ESCAPE]==SDL_PRESSED)
                done=1;
}

int GameManager::Done()
{
        return done;
}

void GameManager::DrawInfo()
{
        copter->DrawInfo(text2d);
}
