#ifndef _GAMEMANAGER_H

#define _GAMEMANAGER_H 1

using namespace std;

#include "camera.h"
#include "landscape.h"
#include "text2d.h"
#include "metamesh.h"
#include <GL/gl.h>
#include <map>
#include "copter.h"
#include "fpscounter.h"

class GameManager
{
        private:
                Camera *camera; /**< current state of camera */
                int cameraView; /**< kind of camera view */
                Landscape *landscape; /**< pointer to landscape */
                Text2D *text2d; /**< pointer to font */
                FpsCounter *fpscounter; /**< pointer to frame per second
                counter */
                int ScreenX; /**< width of screen (window) */
                int ScreenY; /**< height of screen (window) */
                double ViewDistance; /**< distance of view (depth) */
                GLuint LandscapePrimitive; /**< primitive used to draw
                landscape */
                string LocalDataDirectory; /**< name of directory with data */
                string TexturesDirectory; /**< name of directory with textures
                */
                map<string,MetaMesh> MetaMeshes; /**< map of meshes */
                Copter *copter; /**< pointer to controlable copter */
                int done; /**< is game done? */
        public:
                GameManager(int _ScreenX,int _ScreenY, int ScreenBPP);
                ~GameManager();

                void Set3DMode(float ViewAngle, float Near, float Far);
                /**< switches OpenGL projection matrix to 3D mode */

                void Set2DMode();
                /**< switches OpenGL projection matrix to 2D mode */

                void GLCheck();
                /**< displays info about OpenGL implementation/hardware */

                void StartFrame();
                /**< should be called at beginning of frame */

                void EndFrame();
                /**< should be called at end of frame */

                void SetGLCamera();
                /**< sets modelview matrix with camera transformation */

                void LoadLandscape(string filename, double _vd, int MaxTriangles, int PatchLevel);
                /**< initializes landscape */

                void DrawLandscape(double Quality);
                /**< updates and draws landscape */

                void Print(double x, double y, string s);
                /**< prints strings on given position */

                int GetPatchesX();
                int GetPatchesZ();

                void PrintCameraEye();
                void PrintQueuesSize();

                void InitCopter(string filename);
                /**< initializes copter */

                void DrawCopter();
                /**< draws copter */

                void CopterStep();
                /**< updates copter state */

                void FollowWithCamera();
                /**< updates camera */

                void FrontCameraView();
                void EternalCameraView();
                void DistantCameraView();
                void SideCameraView();

                void SwitchCameraView();
                /**< switches kind of camera with keys */

                void SetLight();
                /**< sets OpenGL light */

                void KeysStep();
                /**< check important keys */

                int Done();
                /**< returns 1 if game is done */

                void DrawInfo();
                /**< draws information on screen */

                void UpdateFpsCounter();
                /**< updates and displays frames per second counter */
};

#endif
