#include "heightmap.h"
#include <SDL.h>
#include <cmath>
#include <iostream>
#include "image.h"

HeightMap::HeightMap(string filename, double _scalexz, double _scaley)
{
        unsigned char b;
        double h;
        int i,j,k;
        Image *image;
        cout <<"new HeightMap\n";
        image=new Image(filename); // 8 bit
        HeightMapSizeX=image->GetWidth();
        HeightMapSizeZ=image->GetHeight();
        heights=new float * [HeightMapSizeX];
        gradients_x=new float * [HeightMapSizeX];
        gradients_z=new float * [HeightMapSizeX];
        ScaleXZ=_scalexz;
        ScaleY=_scaley;
        for (i=0;i<HeightMapSizeX;i++)
        {
                heights[i]=new float [HeightMapSizeZ];
                gradients_x[i]=new float [HeightMapSizeZ];
                gradients_z[i]=new float [HeightMapSizeZ];
        }
        cout<<sizeof(float)*HeightMapSizeX*HeightMapSizeZ<<" bytes allocated for float arrays\n";
        for (i=0;i<HeightMapSizeX;i++)
        {
                for (j=0;j<HeightMapSizeZ;j++)
                {
                        b=image->GetPixel(i,j);
                        h=b/256.0;
                        heights[i][j]=h;
                }
        }
        for (i=0;i<HeightMapSizeX;i++)
        {
                for (j=0;j<HeightMapSizeZ;j++)
                {
                        k = i+1;
                        if (k>=HeightMapSizeX) k = 0;
                        gradients_x[i][j]= ScaleY*(heights[k][j] -  heights[i][j])/ScaleXZ;
                        k = j+1;
                        if (k>=HeightMapSizeZ) k = 0;
                        gradients_z[i][j]= ScaleY*(heights[i][k] -  heights[i][j])/ScaleXZ;
                }
        }

        delete image;

        SizeX=(HeightMapSizeX)*ScaleXZ;
        SizeZ=(HeightMapSizeZ)*ScaleXZ;
        //cout <<"ScaleXZ="<<ScaleXZ<<" ScaleY="<<ScaleY<<" SizeX="<<SizeX<<" SizeZ="<<SizeZ<<"\n";
}

HeightMap::~HeightMap()
{
        int i;
        for (i=0;i<HeightMapSizeX;i++)
        {
                delete [] heights[i];
                delete [] gradients_x[i];
                delete [] gradients_z[i];
        }
        delete [] heights;
        delete [] gradients_x;
        delete [] gradients_z;
}

double HeightMap::GetHeightFromMap(int x, int z)
{
        int xx,zz;
        if (x<0)
                xx=x+HeightMapSizeX;
        else
                if (x>=HeightMapSizeX)
                        xx=x-HeightMapSizeX;
                else
                        xx=x;
        if (z<0)
                zz=z+HeightMapSizeZ;
        else
                if (z>=HeightMapSizeZ)
                        zz=z-HeightMapSizeZ;
                else
                        zz=z;
        return ScaleY*heights[xx][zz];
}

double HeightMap::GetHeight(double x, double z)
{
        // TODO linear interpolation
        double xx,zz;
        int xi,zi;
        double xf, zf;
        double h,hx,hz;
        double final_h;
        double dx,dz;
        xx=x/ScaleXZ;
        zz=z/ScaleXZ;
        xf=floor(xx);
        zf=floor(zz);
        xi=(int)xf;
        zi=(int)zf;
        dx=xx-xf;
        dz=zz-zf;
        h=GetHeightFromMap(xi,zi);
        hx=GetHeightFromMap(xi+1,zi);
        hz=GetHeightFromMap(xi,zi+1);
        final_h=(h*(1-dx)+hx*dx+h*(1-dz)+hz*dz)/2.0;
        return final_h;
}

void HeightMap::GetGradientsFromMap(int x, int z,double *nx, double *nz)
{
        int xx,zz;
        if (x<0)
                xx=x+HeightMapSizeX;
        else
                if (x>=HeightMapSizeX)
                        xx=x-HeightMapSizeX;
                else
                        xx=x;
        if (z<0)
                zz=z+HeightMapSizeZ;
        else
                if (z>=HeightMapSizeZ)
                        zz=z-HeightMapSizeZ;
                else
                        zz=z;
        *nx=gradients_x[xx][zz];
        *nz=gradients_z[xx][zz];
}

void HeightMap::GetGradients(double x, double z,double *nx, double *nz)
{
        double xx,zz;
        double xf,zf;
        int xi,zi;
        double nnx,nnz,nxx,nxz,nzx,nzz;
        double dx,dz;
        xx=x/ScaleXZ;
        zz=z/ScaleXZ;
        xf=floor(xx);
        zf=floor(zz);
        xi=(int)xf;
        zi=(int)zf;
        dx=xx-xf;
        dz=zz-zf;
        GetGradientsFromMap(xi,zi,&nnx,&nnz);
        GetGradientsFromMap(xi+1,zi,&nxx,&nxz);
        GetGradientsFromMap(xi,zi+1,&nzx,&nzz);
        *nx=(nnx*(1-dx)+nxx*dx+nnx*(1-dz)+nzx*dz)/2.0;
        *nz=(nnz*(1-dx)+nxz*dx+nnz*(1-dz)+nzz*dz)/2.0;
}

void HeightMap::GetHeightAndGradient(double x, double z, double *h, double *nx,double *nz)
{
        *h=GetHeight(x,z);
        GetGradients(x,z,nx,nz);
}

int HeightMap::Inside(double x, double z)
{
        if ((x<0.0) || (z<0.0))
                return 0;
        if (x>=(HeightMapSizeX+1)*ScaleXZ)
                return 0;
        if (z>=(HeightMapSizeZ+1)*ScaleXZ)
                return 0;
        return 1;
}

double HeightMap::GetSizeX()
{
        return SizeX;
}

double HeightMap::GetSizeZ()
{
        return SizeZ;
}

int HeightMap::GetHeightMapSizeX()
{
        return HeightMapSizeX;
}

int HeightMap::GetHeightMapSizeZ()
{
        return HeightMapSizeZ;
}

double HeightMap::GetScaleXZ()
{
        return ScaleXZ;
}

double HeightMap::GetScaleY()
{
        return ScaleY;
}

void HeightMap::SetLight(Vector3D _LightDirection, Vector3D _ShadowStep, double _LightHeight, double _LightDistance, int _MaxShadowLength)
{
        LightDirection=_LightDirection;
        ShadowStep=_ShadowStep;
        LightHeight=_LightHeight;
        LightDistance=_LightDistance;
        MaxShadowLength=_MaxShadowLength;
}

double HeightMap::Intersect(Vector3D iv, double nx, double nz)
{
        double h,d;
        double InitialHeight, RelativeLightHeight;
        double lx,lz;
        int steps;
        Vector3D v;
        v = iv;
        InitialHeight=iv.y;
        RelativeLightHeight=LightHeight-InitialHeight;
        steps=0;
        d=0.0;
        while (steps<MaxShadowLength)
        {
                v -= ShadowStep;
                d += ScaleXZ;
                h = InitialHeight+(d*RelativeLightHeight) / LightDistance;
                if (GetHeight(v.x,v.z) > h)
                        return 0.0;
                steps++;
        }
        lx=1-fabs(LightDirection.x-nx);
        lz=1-fabs(LightDirection.z-nz);
        if ((lx>0.0) && (lz>0.0))
                return lx*lz;
        else
                return 0.0;
}
