#ifndef _HEIGHTMAP_H

#define _HEIGHTMAP_H 1

using namespace std;

#include <string>
#include "vector3d.h"

class HeightMap
{
        private:
                float **heights; /**< matrix of heights */
                float **gradients_x; /**< matrix of x gradients */
                float **gradients_z; /**< matrix of z gradients */
                int HeightMapSizeX; /**< x size of matrix of heights */
                int HeightMapSizeZ; /**< z size of matrix of heights */
                double ScaleXZ; /**< distance between points in matrix */
                double ScaleY; /**< original height data (0.0-1.0) is
                multiplied by ScaleY */
                double SizeX; /**< x size of heightmap in real world, i.e.
                multiplied by ScaleXZ */
                double SizeZ; /**< z size of heightmap in real world, i.e.
                multiplied by ScaleXZ */
                Vector3D LightDirection; /**< direction of light (sun) */
                Vector3D ShadowStep; /**< direction of casted shadow */
                double LightHeight; /**< height of light (sun) */
                double LightDistance; /**< constant distance from light (sun) */
                int MaxShadowLength; /**< how long shadow can be */
        public:
                HeightMap(string filename, double _scalexz, double _scaley);
                ~HeightMap();

                double GetHeightFromMap(int x, int z);
                /**< returns true height from matrix location (x,z) */

                double GetHeight(double x, double z);
                /**< returns interpolated height from real location (x,z) */

                void GetGradientsFromMap(int x, int z,double *nx, double *nz);
                /**< returns true gradients from matrix location (x,z) */

                void GetGradients(double x, double z,double *nx, double *nz);
                /**< returns interpolated gradients from real location (x,z) */

                void GetHeightAndGradient(double x, double z, double *h, double *nx, double *nz);
                /**< returns interpolated height and interpolated gradients
                 * from real location (x,z) */

                int Inside(double x, double z);
                /**< returns 1 if (x,z) is inside bounds of map */

                double GetSizeX(); /**< returns x size of map */
                double GetSizeZ(); /**< returns z size of map */
                int GetHeightMapSizeX(); /**< returns x size of matrix */
                int GetHeightMapSizeZ(); /**< returns z size of matrix */
                double GetScaleXZ(); /**< returns xz scale */
                double GetScaleY(); /**< returns y scale */

                double Intersect(Vector3D iv, double nx, double nz);
                /**< returns light intensity in given point iv with gradient (nx,nz) */

                void SetLight(Vector3D _LightDirection, Vector3D _ShadowStep, double _LightHeight, double _LightDistance, int _MaxShadowLength);
                /**< sets light (sun) parameters */
};

#endif
