#include "image.h"
#include <SDL_image.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <cassert>
#include <iostream>

Image::Image(string filename)
{
        surface=IMG_Load(filename.c_str());
        if (surface==NULL)
        {
                cout <<"couldn't open image: "<<filename<<"\n";
                cout <<"make sure you installed and generated textures\n";
                exit(-1);
        }
        pixels=(Uint8*)surface->pixels;
        bpp=surface->format->BytesPerPixel;
        pitch=surface->pitch;
}

Image::~Image()
{
        SDL_FreeSurface(surface);
}

int Image::GetWidth()
{
        assert(surface);
        return surface->w;
}

int Image::GetHeight()
{
        assert(surface);
        return surface->h;
}

Uint8 Image::GetPixel(int x, int y)
{
        return pixels[y*pitch+x];
}

void Image::GetRGBPixel(int x, int y, Uint8 *r, Uint8 *g, Uint8 *b)
{
        *r=pixels[bpp*x+pitch*y];
        *g=pixels[bpp*x+pitch*y+1];
        *b=pixels[bpp*x+pitch*y+2];
}

GLuint Image::GenerateTexture()
{
        GLuint id;
        GLuint result;
        if (surface->format->BytesPerPixel!=3)
        {
                cout <<"image in bad format\n";
                exit(-1);
        }
        if (surface->pitch!=surface->w*3)
        {
                cout <<"image in bad format (pitch!=w)\n";
                exit(-1);
        }
        glGenTextures(1,&id);
        glBindTexture(GL_TEXTURE_2D,id);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
        result=gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,surface->w, surface->h,GL_RGB,GL_UNSIGNED_BYTE,surface->pixels);
        if (result!=0)
        {
                cout <<"problem building Mipmaps\n";
                exit(-1);
        }
        return id;
}

