#ifndef _LANDSCAPE_H

#define _LANDSCAPE_H 1

using namespace std;

#include "heightmap.h"
#include "bttpool.h"
#include "patch.h"
#include "splitqueue.h"
#include "mergequeue.h"
#include "landscapetexture.h"
#include <string>
#include "indicestree.h"

#define MaxIndices 20000

class Landscape
                        /** Landscape in game - heights, patches and ROAM queues. */
{
        private:
                HeightMap *hm; /**< heights are stored here */
                BTTPool *bp; /**< pool of free bttnodes */
                Patch ***patches; /**< matrix of patches */
                double SizeX; /**< x size of map */
                double SizeZ; /**< z size of map */
                int PatchesX; /**< x size of patches matrix */
                int PatchesZ; /**< z size of patches matrix */
                double PatchSize; /**< length/width of patch */
                int MaxQsSize; /**< how big Qs is */
                int MaxQmSize; /**< how big Qm is */
                int BTTPoolSize; /**< how big bttpool is */
                SplitQueue *Qs; /**< ROAM split queue */
                MergeQueue *Qm; /**< ROAM merge queue */
                int CalcXInPatches(int x); /**< wrap x value to fit in patches
                matrix */
                int CalcZInPatches(int z); /**< wrap z value to fit in patches
                matrix */
                void LoadHeightMap(string filename); /**< initializes height
                map */
                Frustum frustum; /**< OpenGL frustum of view */
                double ViewDistance; /**< distance of view */
                int PatchLevel; /**< how big is patch, how much it one can
                split it */
                int TextureSize; /**< size of texture for patch */
                int MaxTriangles; /**< how many triangles could be in ROAM */
                IndicesTree *LeftIndicesTree; /**< indices for left side of
                patch */
                IndicesTree *RightIndicesTree; /**< indices for right side of
                patch */
                GLuint *indices; /**< temporary array used in patches */
                Vector3D LightOrigin; /**< position of light (sun) */
                Vector3D LightDirection; /**< direction of light (sun) */
                double LightDistance; /**< constant distance of light (sun) */
        public:
                Landscape(string filename, int _PatchLevel, double _vd, int _TextureSize, int _MaxTriangles);
                ~Landscape();

                void Draw(Vector3D position,GLuint primitive);
                /**< draw patches around given position, with given primitive
                 * */
                void DrawPatches(int x1, int z1, int x2, int z2,GLuint primitive);
                /**< draw patches in rectangle (x1,z1,x2,z2) with given
                 * primitive */

                int GetPatchesX();
                double GetSizeX();
                double GetSizeZ();
                int GetPatchesZ();

                void Update(Vector3D *CameraEye, double TargetPriority);
                /**< update ROAM queues */

                void AddSplitedTriangleChildren(LandscapeTriangle *lt, Vector3D *CameraEye);
                /**< create children of lt, add them to Qs, remove lt from Qs */

                void ForceSplit(BTTNode *tree,Vector3D *CameraEye);
                /**< split diamond, if level is different than base neighbour
                 * level, then split neighbour first */

                void SplitPhase(Vector3D *CameraEye);
                /**< force split triangle with highest priority */

                void AddNewMergeableDiamond(BTTNode *tree, Vector3D *CameraEye);
                /**< check if new diamond is available, if yes - add it to Qm */

                void MergePhase(Vector3D *CameraEye);
                /**< merge diamond with highest priority */

                string GetQueuesSizeString();
                /**< returns string with size of queues */

                double GetHeight(double x, double z);
                /**< returns interpolated height of real location (x,z) */

                void GenerateTextures(string filename, string LocalDataDirectory, string TexturesDirectory, double LightAmbient, double LightEnergy, int MaxShadowLength);
                /**< generate textures for whole landscape */

                void GenerateTexture(double x1, double z1, double x2, double z2, LandscapeTexture **textures, int ntextures, string filename, double LightAmbient, double LightEnergy);
                /**< generate and save to file texture for one given patch */

                string CreateTextureFileName(string dir, int x, int z);
                /**< returns correct filename for texture for given patch */

                void CreatePatches(string LocalDataDirectory);
                /**< initializes patches */

                Vector3D GetLightDirection();
                /**< returns direction of light */
};

#endif
