#ifndef _MATRIX4_H

#define _MATRIX4_H 1

#include "vector3d.h"

class Matrix4
                        /** Matrix 4x4 */
{
        private:
                double e[4][4];
                /**< rows are first, columns are second */
        public:
                Matrix4 ();
                Matrix4 (const Matrix4 &m);
                Matrix4 (Vector3D &v);
                ~Matrix4 ();

                void LoadIdentity(); /**< sets identity matrix */

                void CreateRotateXMatrix (double angle);
                void CreateRotateYMatrix (double angle);
                void CreateRotateZMatrix (double angle);
                void CreateTranslateMatrix (Vector3D v);
                void CreateScaleMatrix (Vector3D v);
                void CreateRotateMatrix (Vector3D v);
                void CreateCameraMatrix(Vector3D e, Vector3D p, Vector3D u);

                Vector3D operator * (Vector3D v);
                Matrix4 operator * (Matrix4 m);
                Matrix4 operator + (Matrix4 m);
                Matrix4 operator * (double a);
                Matrix4 operator / (double a);
                void operator *= (Matrix4 m);
                friend Vector3D operator * (Vector3D v1, Matrix4 m);

                void Negate(); /**< changes sign of every element */
                Matrix4 Transpose(); /**< exchanges columns and rows */
                double Cofactor(int x, int y);
                /**< returns determinant of matrix without given column and row
                 * */ 
                double Determinant();
                /**< returns determinant of matrix */
                Matrix4 Adjoint();
                /**< returns adjointed matrix */
                Matrix4 Invert();
                /**< returns inverted matrix */

                void Zero (); /**< sets all elements to 0.0 */

                void Print (); /**< prints all elements */

                void glLoadMatrix(); /**< load matrix to OpenGL */
                void glMultMatrix(); /**< multiplies current OpenGL matrix */
                void glGetProjectionMatrix(); /**< gets OpenGL projection
                matrix */
                void glGetModelviewMatrix(); /**< gets OpenGL modelview matrix
                */

                void AddColumns(int c1, int c2, double *a);
                /**< adds columns c1 and c2, save result to a */

                void SubColumns(int c1, int c2, double *a);
                /**< subs columns c1 and c2, save result to b */
};

#endif
