#include "mergequeue.h"
#include <iostream>

#define LandscapeUpdateArrays 100

MergeQueue::MergeQueue(int _maxsize, double _maxx, double _maxz, double _vd)
{
        int i;
        cout <<"new MergeQueue\n";
        maxsize=_maxsize;
        elements=new MergeableDiamond * [maxsize];
        Pool=new MergeableDiamond * [maxsize];
        cout <<sizeof(MergeableDiamond*)*maxsize*2<<" bytes allocated for pointer Arrays\n";
        PoolSize=maxsize;
        size=0;
        maxx=_maxx;
        maxz=_maxz;
        for (i=0;i<maxsize;i++)
                Pool[i]=new MergeableDiamond();
        cout <<sizeof(MergeableDiamond)*maxsize<<" bytes allocated for Pool\n";

        NumberOfArrays=LandscapeUpdateArrays;
        SizeOfArrays=maxsize/(LandscapeUpdateArrays-1)+1;
        Arrays=new MergeableDiamond ** [NumberOfArrays];
        Sizes=new int [NumberOfArrays];
        for (i=0;i<NumberOfArrays;i++)
        {
                Arrays[i]=new MergeableDiamond *[SizeOfArrays];
                Sizes[i]=0;
        }
        cout <<sizeof(MergeableDiamond **)*NumberOfArrays+sizeof(int)*NumberOfArrays+sizeof(MergeableDiamond *)*SizeOfArrays*NumberOfArrays<<" bytes allocated for LandscapeUpdateArray\n";
        CurrentArray=0;

        ViewDistance=_vd;
}

MergeQueue::~MergeQueue()
{
        int i;
        cout <<"delete MergeQueue\n";
        cout <<"(free elements)\n";
        while (!Empty())
        {
                Remove(1);
        }
        cout <<"(pool array)\n";
        for (i=0;i<maxsize;i++)
        {
                delete Pool[i];
        }
        cout <<"(pool)\n";
        delete [] Pool;
        cout <<"(elements)\n";
        delete [] elements;
        cout <<"(updatearray)\n";
        for (i=0;i<NumberOfArrays;i++)
                delete [] Arrays[i];
        delete [] Arrays;
        delete [] Sizes;
}

MergeableDiamond *MergeQueue::AllocateDiamond(BTTNode *_first, BTTNode *_second, TriangleInfoTree *_ft, int _fn, TriangleInfoTree *_st, int _sn)
{
        MergeableDiamond *md;
        PoolSize--;
        md=Pool[PoolSize];
        md->FirstBTTNode=_first;
        md->SecondBTTNode=_second;
        md->First_it=_ft;
        md->Second_it=_st;
        md->First_it_node=_fn;
        md->Second_it_node=_sn;
        return md;
}

MergeableDiamond *MergeQueue::GetDiamond(BTTNode *bttnode)
{
        return elements[bttnode->QueuePos-1];
}

void MergeQueue::Add(MergeableDiamond *element, Vector3D *CameraEye)
{
        int current;
        current=size+1;
        elements[current-1]=element;

        element->FirstBTTNode->QueuePos=current;
        element->SecondBTTNode->QueuePos=current;

        size++;
        if (size>maxsize)
        {
                cout <<"ooops - MergeQueue::Add size>=maxsize!\n";
                exit(-1);
        }
        element->CalcPriority(CameraEye, maxx, maxz, ViewDistance);
        Fix(current);

        int n,n1,n2;
        int s1,s2;
        do
        {
                n1=rand()%NumberOfArrays;
                n2=rand()%NumberOfArrays;
                s1=Sizes[n1];
                s2=Sizes[n2];
                if (s1<s2)
                {
                        n=n1;
                }
                else
                {
                        n=n2;
                }
        } while (Sizes[n]>=SizeOfArrays);
        Arrays[n][Sizes[n]]=element;
        element->LandscapeUpdateArray_nr=n;
        element->LandscapeUpdateArray_pos=Sizes[n];
        Sizes[n]++;
}

void MergeQueue::Remove(int pos)
{
        int n,p,s;
        n=elements[pos-1]->LandscapeUpdateArray_nr;
        p=elements[pos-1]->LandscapeUpdateArray_pos;
        s=Sizes[n]-1;
        if (p!=s)
        {
                Arrays[n][p]=Arrays[n][s];
                Arrays[n][p]->LandscapeUpdateArray_nr=n;
                Arrays[n][p]->LandscapeUpdateArray_pos=p;
        }
        Sizes[n]--;

        Pool[PoolSize]=elements[pos-1];
        PoolSize++;
        elements[pos-1]=elements[size-1];
        size--;
        if (pos<=size)
        {
                elements[pos-1]->FirstBTTNode->QueuePos=pos;
                elements[pos-1]->SecondBTTNode->QueuePos=pos;
        }
        if (pos<=size)
                Fix(pos);
}

void MergeQueue::RecalcPriorities(Vector3D *CameraEye)
{
        int i,current,ActiveElements;
        MergeableDiamond **elementspointers;

        ActiveElements=0;

        CurrentArray++;
        if (CurrentArray>=NumberOfArrays)
        {
                CurrentArray=0;
        }

        elementspointers=Arrays[CurrentArray];
        ActiveElements=Sizes[CurrentArray];

        //    cout <<"Active Elements:"<<ActiveElements<<"\n";
        for (i=0;i<ActiveElements;i++)
        {
                elementspointers[i]->CalcPriority(CameraEye, maxx, maxz, ViewDistance);
                current=elementspointers[i]->FirstBTTNode->QueuePos;
                Fix(current);
        }
}

int MergeQueue::Empty()
{
        if (size==0)
                return 1;
        else
                return 0;
}

int MergeQueue::GetSize()
{
        return size;
}

void MergeQueue::Fix(int pos)
{
        int parent,child;
        MergeableDiamond *tmp;
        int l;
        float p1, p2;
        parent=pos>>1;
        if (parent>0)
        {
                if (elements[parent-1]->priority<elements[pos-1]->priority)
                {
                        while (elements[pos-1]->priority>elements[parent-1]->priority)
                        {
                                tmp=elements[pos-1];
                                elements[pos-1]=elements[parent-1];
                                elements[parent-1]=tmp;

                                elements[pos-1]->FirstBTTNode->QueuePos=pos;
                                elements[pos-1]->SecondBTTNode->QueuePos=pos;
                                elements[parent-1]->FirstBTTNode->QueuePos=parent;
                                elements[parent-1]->SecondBTTNode->QueuePos=parent;

                                pos=parent;
                                parent=pos>>1;
                                if (parent==0)
                                        return;
                        }
                        return;
                }
        }
        while (1)
        {
                child=pos<<1;
                if (child>size)
                        return;
                p1=elements[child-1]->priority;
                l=pos;
                if (p1>elements[pos-1]->priority)
                {
                        l=child;
                }
                child++;
                if (child<=size)
                {
                        p2=elements[child-1]->priority;
                        if ((p2>elements[pos-1]->priority) && (p2>p1))
                        {
                                l=child;
                        }
                };
                if (l!=pos)
                {
                        tmp=elements[pos-1];
                        elements[pos-1]=elements[l-1];
                        elements[l-1]=tmp;
                        elements[pos-1]->FirstBTTNode->QueuePos=pos;
                        elements[pos-1]->SecondBTTNode->QueuePos=pos;
                        elements[l-1]->FirstBTTNode->QueuePos=l;
                        elements[l-1]->SecondBTTNode->QueuePos=l;
                        pos=l;
                }
                else
                        return;
        }
}

void MergeQueue::RemoveBTTNodeFromQm(BTTNode *bttnode)
{
        int pos;
        MergeableDiamond *md;
        BTTNode *t1, *t2;
        pos=bttnode->QueuePos;
        if (pos!=NOT_IN_QUEUE)
        {
                md=(MergeableDiamond*)elements[pos-1];
                t1=md->FirstBTTNode;
                t2=md->SecondBTTNode;
                Remove(pos);
                t1->QueuePos=NOT_IN_QUEUE;
                t2->QueuePos=NOT_IN_QUEUE;
        }
}

float MergeQueue::GetHighestPriority()
{
        if (size==0)
                return 0.0;
        else
                return elements[0]->priority;
}

MergeableDiamond *MergeQueue::GetHighest()
{
        MergeableDiamond *e;
        e=(MergeableDiamond*)elements[0];
        return e;
}

