#ifndef _MERGEQUEUE_H 

#define _MERGEQUEUE_H 1

#include "mergeablediamond.h"
#include "bttnode.h"

class MergeQueue
                        /**< Queue of mergeable diamonds. */
{
        protected:
                int maxsize; /**< max size of queue */
                int size; /**< current size of queue */
                MergeableDiamond **elements; /**< pointers to mergeable
                				  diamonds, one element is just
                				  a pointer, pointers are
                				  placed in array when new
                				  diamond is added to queue */
                MergeableDiamond **Pool; /**< pool of free diamonds */
                MergeableDiamond ***Arrays; /**< update arrays */
                int *Sizes; /**< size of every update array */
                int PoolSize; /**< size of pool */
                double maxx; /**< x size of landscape */
                double maxz; /**< z size of landscape */
                double ViewDistance; /**< distance of view */
                int NumberOfArrays; /**< how many update arrays exist */
                int SizeOfArrays; /**< size of every update array */
                int CurrentArray; /**< update array which is update now */
        public:
                MergeQueue(int _maxsize, double _maxx, double _maxz, double _vd);
                ~MergeQueue();

                void Add(MergeableDiamond *element, Vector3D *CameraEye);
                /**< add given element to queue */

                void Remove(int pos);
                /**< remove element from given position from queue */

                int Empty();
                /**< returns 1 if queue is empty */

                int GetSize();
                /**< returns size of queue */

                void Fix(int pos);
                /**< fix tree, starts at given position */

                void RecalcPriorities(Vector3D *CameraEye);
                /**< recalculates priorities in tree */

                MergeableDiamond *GetDiamond(BTTNode *bttnode);
                /**< returns diamond containing given bttnode */

                MergeableDiamond *AllocateDiamond(BTTNode *_first, BTTNode *_second, TriangleInfoTree *_ft, int _fn, TriangleInfoTree *_st, int _sn);
                /**< allocates new diamond */

                void RemoveBTTNodeFromQm(BTTNode *bttnode);
                /**< removes given bttnode from queue */

                MergeableDiamond *GetHighest();
                /**< returns diamond with highest priority */

                float GetHighestPriority();
                /**< returns highest priority */
};

#endif
