#ifndef _MESH_H

#define _MESH_H 1

#include "vector3d.h"
#include "point3d.h"
#include "face.h"
#include "matrix4.h"
#include <string>
#include <fstream>
#include <GL/gl.h>

class Mesh
                        /** Mesh is 3D space. */
{
        protected:
                string name; /**< name of mesh */

                int npoints; /**< number of points in mesh */
                Point3D **points; /**< array of points in mesh */

                int nfaces; /**< number of faces in mesh */
                Face **faces; /**< array of faces in mesh */

                Vector3D Loc; /**< Loc from Blender - translation vector of mesh */
                Vector3D Rot; /**< Rot from Blender - rotation vector of mesh */
                Vector3D Size; /**< Size from Blender - scale vector of mesh */

                GLint list; /**< identifier of OpenGL list */

                double BoundingSphereRadius; /**< radius of bounding sphere of mesh */

        public:
                Mesh();
                Mesh (ifstream * f); /**< loads mesh from textfile generated
                			  with Blender export script */
                ~Mesh ();

                void MakeTree(); /**< generates mesh with 2-faced "tree" */
                void LoadFaces (ifstream * f);
                void CalcNormals (); /**< calculates normal for every point */

                void GenerateList(); /**< generates OpenGL list with
                			  instructions to draw mesh */
                void Draw (); /**< draws generated list */
                void DrawNormals (); /**< draws normals (as lines) to all points */

                string GetName (); /**< returns name of mesh */
                void GetMinMax (Vector3D * rmin, Vector3D * rmax); /**< sets rmin and rmax
                vectors to smaller and
                larger values of x,y,z */
                double GetBoundingSphereRadius(); /**< returns radius of bounding sphere */
                Vector3D GetLoc(); /**< returns Loc vector */

                Point3D *RandomPoint (); /**< returns random point from mesh */


                void FindBoundingSphereRadius(); /**< calculates radius of bouding sphere */
                Vector3D FindCenter(); /**< returns position of center of mesh */

                int CheckCrash(Vector3D ContactPoint); /**< compares length of
                					    ContactPoint with
                					    radius of bounding
                					    sphere, returns 1
                					    if crash happen */
};

#endif
