#include "metamesh.h"
#include <assert.h>
#include <iostream>
#include <fstream>

MetaMesh::~MetaMesh ()
{
}

void MetaMesh::CalcNormals ()
{
        int i;
        for (i = 0; i < nmeshes; i++)
                meshes[i]->CalcNormals ();
}

MetaMesh::MetaMesh()
{
        nmeshes=0;
        meshes=NULL;
}

MetaMesh::MetaMesh (string filename)
{
        ifstream *f;
        string s;
        int i;

        f = new ifstream (filename.c_str());
        if (f == NULL)
        {
                cout <<"can't open file: "<<filename<<"\n";
                exit (-1);
        }
        (*f)>>s;
        if (s!="DecopterMetaMesh")
        {
                cout <<filename<<" is not DecopterMeshesFile\n";
                exit (-1);
        }

        (*f)>>s;
        if (s!="meshes")
        {
                cout <<"error: "<<s<<" expected: meshes\n";
                exit (-1);
        }

        (*f)>>nmeshes;
        cout <<"loading "<<nmeshes<<" meshes from file "<<filename<<"\n";
        meshes = new Mesh *[nmeshes];
        for (i = 0; i < nmeshes; i++)
        {
                meshes[i] = new Mesh (f);
        }
        (*f)>>s;
        if (s!="EndOfMeshes")
        {
                cout <<"error: "<<s<<" expected: EndOfMeshes\n";
                exit (-1);
        }
        f->close();
        FindBoundingSphereRadius();
}

int MetaMesh::GetNMeshes()
{
        return nmeshes;
}

Mesh *MetaMesh::GetMesh(int i)
{
        assert(i<nmeshes);
        return meshes[i];
}

Mesh *MetaMesh::GetMesh(string n)
{
        int i;
        for (i=0;i<nmeshes;i++)
        {
                if (meshes[i]->GetName()==n)
                        return meshes[i];
        }
        cout <<"MetaMesh::GetMesh - couldn't find mesh: "<<n<<"\n";
        exit(-1);
        //	return NULL;
}

void MetaMesh::GenerateLists()
{
        int i;
        for (i=0;i<nmeshes;i++)
        {
                meshes[i]->GenerateList();
        }
}

void MetaMesh::FindBoundingSphereRadius()
{
        int i;
        double d;
        Vector3D v;
        BoundingSphereRadius=0.0;
        for (i=0;i<nmeshes;i++)
        {
                v=meshes[i]->GetLoc();
                d=v.Length()+meshes[i]->GetBoundingSphereRadius();
                if (d>BoundingSphereRadius)
                        BoundingSphereRadius=d;
        }
}

double MetaMesh::GetBoundingSphereRadius()
{
        return BoundingSphereRadius;
}
