#include "metaobject3d.h"
#include <GL/gl.h>
#include <iostream>
#include "common.h"

MetaObject3D::MetaObject3D(MetaMesh *meshes)
{
        int i;
        Object3D *object;
        string name;
        for (i=0;i<meshes->GetNMeshes();i++)
        {
                object=new Object3D(meshes->GetMesh(i));
                name=meshes->GetMesh(i)->GetName();
                objects[name]=object;
        }
}

MetaObject3D::~MetaObject3D()
{
}

void MetaObject3D::Draw()
{
        map<string,Object3D*>::const_iterator i;
        glColor3f(1,1,1);
        glPushMatrix ();
        matrix.glMultMatrix ();
        for (i=objects.begin();i!=objects.end();i++)
                i->second->Draw ();
        glPopMatrix ();
}

void MetaObject3D::SetTextures(string filename)
{
        string mesh_name, texture_filename;
        Image *image;
        map<string,Object3D*>::const_iterator i;
        GLuint DefaultTexture;
        ifstream *f = new ifstream (filename.c_str());

        if (f == NULL)
        {
                cout <<"can't open textures file: "<<filename<<"\n";
                exit (-1);
        }
        string TexturesDirectory = GetTexturesDirectory();

        Expect(f,"default");
        (*f)>>texture_filename;
        image=new Image(TexturesDirectory+texture_filename);
        DefaultTexture=image->GenerateTexture();
        for (i=objects.begin();i!=objects.end();i++)
                i->second->SetTexture(DefaultTexture);
        delete image;

        while (!f->eof())
        {
                (*f)>>mesh_name;
                (*f)>>texture_filename;
                image=new Image(TexturesDirectory+texture_filename);
                SetTexture(mesh_name,image->GenerateTexture());
                delete image;
        }
        f->close();
}

void MetaObject3D::SetTexture(string name, GLuint t)
{
        objects[name]->SetTexture(t);
}

void MetaObject3D::SetMatrix(Matrix4 m)
{
        matrix=m;
}

void MetaObject3D::SetObjectMatrix(string name, Matrix4 m)
{
        objects[name]->SetMatrixWithoutLoc(m);
}
