#ifndef _PATCH_H

#define _PATCH_H 1

#include "bttnode.h"
#include "bttpool.h"
#include "triangleinfotree.h"
#include "heightmap.h"
#include <GL/gl.h>
#include "object3d.h"
#include <list>
#include "box.h"
#include "frustum.h"
#include "splitqueue.h"
#include "indicestree.h"
#include "texturestruct.h"

class Patch
                        /**< Rectangular area on landscape. Every patch is made of two big triangles.
                         * Every triangle can be splitted to smaller ones - if it's needed for detailed
                         * view. Splitting and merging triangles is permormed by LOD method called
                         * ROAM. */
{
        private:
                BTTNode *LeftTree; /**< left tree of bttnodes */
                BTTNode *RightTree; /**< right tree of bttnodes */

                TriangleInfoTree *LeftInfoTree; /**< left triangle info tree */
                TriangleInfoTree *RightInfoTree; /**< right triangle info tree
                */
                TriangleInfoTree *CurrentInfoTree; /**< pointer to current
                triangle info tree */

                double x1; /**< left x coordinate of patch */
                double z1; /**< top z coordinate of patch */
                double x2; /**< right x coordinate of patch */
                double z2; /**< bottom z coordinate of patch */

                int PatchSizeInPoints; /**< width/height patch in points */
                int NumberOfPoints; /**< number of points in
                horizontal/vertical line (i.e. PatchSizeInPoints+1) */

                VertexStruct *Vertices; /**< array of vertices */
                TextureStruct *TexCoords; /**< array of texture coordinates */
                int ArraysSize; /**< size of vertices and texcoords arrays */

                GLuint texture; /**< identifier of OpenGL texture */

                HeightMap *hm; /**< pointer to height map */

                int nindices; /**< how many indices are currently active in
                patch */
                GLuint *indices; /**< pointer to indices array (it's not private) */

                double MinY; /**< minimal height of terrain */
                double MaxY; /**< maximal height of terrain */

                IndicesTree *LeftIndicesTree; /**< pointer to tree of indices
                of left triangle */
                IndicesTree *RightIndicesTree; /**< pointer to tree of indices
                of right triangle */

                int **CurrentIndices; /**< pointer to array of indices */

        public:
                Patch(double _x1, double _z1, double _x2, double _z2, int PatchLevel, HeightMap *_hm, BTTPool *bttpool, int TextureSize, IndicesTree *lit, IndicesTree *rit, GLuint *_indices);
                ~Patch();

                void RenderTree(BTTNode *tree, int node);
                /**< recursive rendering of tree, triangles are saved in
                 * indices array */

                int GenerateIndices();
                /**< generates indices for two trees */

                void Draw(GLuint primitive);
                /**< draws patch with given primitive, indices must be saved
                 * first */

                void SetNeighbours(Patch *ln, Patch *rn, Patch *tn, Patch *bn);
                /**< sets neighbours for patch */

                void FillQs(SplitQueue *Qs);
                /**< puts two big triangles in Qs */

                void CreateArrays(int TextureSize);
                /**< builds vertices and texcoords arrays */

                void FindMinMaxY();
                /**< calculates MinY and MaxY */

                int CheckIfEnabled(double x, double z,Frustum *frustum);
                /**< checks if patch is visible, and sets visible flags in info
                 * tress */

                void SetTexture(GLuint _txt);
                /**< sets OpenGL texture indentifier */

                void Disable();
                /**< sets visible flags in info trees to zero */

};

#endif
