#ifndef _POINT3D_H

#define _POINT3D_H

#include "vector3d.h"

class Point3D
                        /** Represents point located in 3D space, with Normal vector and texture coordinates.*/
{
        private:
                Vector3D Position; /**< location of point in space */
                Vector3D Normal; /**< Normal vector to point */
                float u,v; /**< texture coordinates for point */

        public:
                Point3D (const Vector3D &pos); /**< constructor - set Position */
                Point3D (const Vector3D &pos, float _u, float _v); /**< constructor - set Position, u and v */
                Point3D (const Point3D &p); /**< copy Point3D */

                void Draw (); /**< draw Point3D with OpenGL */
                void DrawNormal(); /**< draw Normal vector as line */

                void AddNormal (Vector3D n); /**< add n to Normal */
                void CalcNormal (); /**< Normalize Normal */

                Vector3D GetPosition (); /**< returns Position */
                Vector3D GetNormal (); /**< returns Normal */

                void SetPosition (Vector3D vec); /**< sets Position to vec */
                void SetNormal (Vector3D vec); /**< sets Normal to vec */
                void SetTexCoord (float _u, float _v); /**< sets texture coordinates to _u and _v*/

                void Print(); /**< print Position, Normal, u and v to cout */
};

#endif
