#include "splitqueue.h"
#include <iostream>

#define LandscapeUpdateArrays 100

SplitQueue::SplitQueue(int _maxsize, double _maxx, double _maxz, double _vd)
{
        int i;
        cout <<"new SplitQueue\n";
        maxsize=_maxsize;
        elements=new LandscapeTriangle * [maxsize];
        Pool=new LandscapeTriangle * [maxsize];
        cout <<sizeof(LandscapeTriangle*)*maxsize*2<<" bytes allocated for pointer Arrays\n";
        PoolSize=maxsize;
        size=0;
        maxx=_maxx;
        maxz=_maxz;
        for (i=0;i<maxsize;i++)
                Pool[i]=new LandscapeTriangle();
        cout <<sizeof(LandscapeTriangle)*maxsize<<" bytes allocated for Pool\n";

        NumberOfArrays=LandscapeUpdateArrays;
        SizeOfArrays=maxsize/(LandscapeUpdateArrays-1)+1;
        Arrays=new LandscapeTriangle ** [NumberOfArrays];
        Sizes=new int [NumberOfArrays];
        for (i=0;i<NumberOfArrays;i++)
        {
                Arrays[i]=new LandscapeTriangle *[SizeOfArrays];
                Sizes[i]=0;
        }
        cout <<sizeof(LandscapeTriangle **)*NumberOfArrays+sizeof(int)*NumberOfArrays+sizeof(LandscapeTriangle *)*SizeOfArrays*NumberOfArrays<<" bytes allocated for LandscapeUpdateArray\n";
        CurrentArray=0;

        ViewDistance=_vd;
}

SplitQueue::~SplitQueue()
{
        int i;
        cout <<"delete SplitQueue\n";
        cout <<"(free elements)\n";
        while (!Empty())
        {
                Remove(1);
        }
        cout <<"(pool array)\n";
        for (i=0;i<maxsize;i++)
        {
                delete Pool[i];
        }
        cout <<"(pool)\n";
        delete [] Pool;
        cout <<"(elements)\n";
        delete [] elements;
        cout <<"(updatearray)\n";
        for (i=0;i<NumberOfArrays;i++)
                delete [] Arrays[i];
        delete [] Arrays;
        delete [] Sizes;
}

LandscapeTriangle *SplitQueue::AllocateTriangle(BTTNode *_bttnode,TriangleInfoTree *_it,int _it_node)
{
        LandscapeTriangle *lt;
        PoolSize--;
        lt=Pool[PoolSize];
        lt->FirstBTTNode=_bttnode;
        lt->First_it=_it;
        lt->First_it_node=_it_node;
        return lt;
}

LandscapeTriangle *SplitQueue::GetTriangle(BTTNode *bttnode)
{
        return elements[bttnode->QueuePos-1];
}

void SplitQueue::Add(LandscapeTriangle *element, Vector3D *CameraEye)
{
        int current;
        current=size+1;
        elements[current-1]=element;
        element->FirstBTTNode->QueuePos=current;
        size++;
        if (size>maxsize)
        {
                cout <<"ooops - SplitQueue::Add size>=maxsize!\n";
                exit(-1);
        }
        element->CalcPriority(CameraEye, maxx, maxz, ViewDistance);
        Fix(current);

        int n,n1,n2;
        int s1,s2;
        do
        {
                n1=rand()%NumberOfArrays;
                n2=rand()%NumberOfArrays;
                s1=Sizes[n1];
                s2=Sizes[n2];
                if (s1<s2)
                {
                        n=n1;
                }
                else
                {
                        n=n2;
                }
        } while (Sizes[n]>=SizeOfArrays);
        Arrays[n][Sizes[n]]=element;

        element->LandscapeUpdateArray_nr=n;
        element->LandscapeUpdateArray_pos=Sizes[n];

        Sizes[n]++;
}

void SplitQueue::Remove(int pos)
{
        int n,p,s;
        n=elements[pos-1]->LandscapeUpdateArray_nr;
        p=elements[pos-1]->LandscapeUpdateArray_pos;
        s=Sizes[n]-1;
        if (p!=s)
        {
                Arrays[n][p]=Arrays[n][s];
                Arrays[n][p]->LandscapeUpdateArray_nr=n;
                Arrays[n][p]->LandscapeUpdateArray_pos=p;
        }
        Sizes[n]--;

        Pool[PoolSize]=elements[pos-1];
        PoolSize++;
        elements[pos-1]=elements[size-1];
        size--;
        if (pos<=size)
                elements[pos-1]->FirstBTTNode->QueuePos=pos;
        if (pos<=size)
                Fix(pos);
}

void SplitQueue::RecalcPriorities(Vector3D *CameraEye)
{
        int i,current,ActiveElements;
        LandscapeTriangle **elementspointers;

        ActiveElements=0;

        CurrentArray++;
        if (CurrentArray>=NumberOfArrays)
        {
                CurrentArray=0;
        }

        elementspointers=Arrays[CurrentArray];
        ActiveElements=Sizes[CurrentArray];

        for (i=0;i<ActiveElements;i++)
        {
                elementspointers[i]->CalcPriority(CameraEye, maxx, maxz, ViewDistance);
                current=elementspointers[i]->FirstBTTNode->QueuePos;
                Fix(current);
        }
}

int SplitQueue::Empty()
{
        if (size==0)
                return 1;
        else
                return 0;
}

int SplitQueue::GetSize()
{
        return size;
}

void SplitQueue::Fix(int pos)
{
        int parent,child;
        LandscapeTriangle *tmp;
        int l;
        float p1, p2;
        parent=pos>>1;
        if (parent>0)
        {
                if (elements[parent-1]->priority<elements[pos-1]->priority)
                {
                        while (elements[pos-1]->priority>elements[parent-1]->priority)
                        {
                                tmp=elements[pos-1];
                                elements[pos-1]=elements[parent-1];
                                elements[parent-1]=tmp;
                                elements[pos-1]->FirstBTTNode->QueuePos=pos;
                                elements[parent-1]->FirstBTTNode->QueuePos=parent;
                                pos=parent;
                                parent=pos>>1;
                                if (parent==0)
                                        return;
                        }
                        return;
                }
        }
        while (1)
        {
                child=pos<<1;
                if (child>size)
                        return;
                p1=elements[child-1]->priority;
                l=pos;
                if (p1>elements[pos-1]->priority)
                {
                        l=child;
                }
                child++;
                if (child<=size)
                {
                        p2=elements[child-1]->priority;
                        if ((p2>elements[pos-1]->priority) && (p2>p1))
                        {
                                l=child;
                        }
                };
                if (l!=pos)
                {
                        tmp=elements[pos-1];
                        elements[pos-1]=elements[l-1];
                        elements[l-1]=tmp;
                        elements[pos-1]->FirstBTTNode->QueuePos=pos;
                        elements[l-1]->FirstBTTNode->QueuePos=l;
                        pos=l;
                }
                else
                        return;
        }
}


void SplitQueue::RemoveBTTNodeFromQs(BTTNode *bttnode)
{
        int pos;
        pos=bttnode->QueuePos;
        Remove(pos);
        bttnode->QueuePos=NOT_IN_QUEUE;
}

float SplitQueue::GetHighestPriority()
{
        if (size==0)
                return 0.0;
        else
                return elements[0]->priority;
}

LandscapeTriangle *SplitQueue::GetHighest()
{
        LandscapeTriangle *e;
        e=(LandscapeTriangle*)elements[0];
        return e;
}

