#ifndef _SPLITQUEUE_H 

#define _SPLITQUEUE_H 1

#include "landscapetriangle.h"
#include "bttnode.h"

class SplitQueue
                        /** Queue of landscape triangles. */
{
        protected:
                int maxsize; /**< max size of queue */
                int size; /**< current size of queue */
                LandscapeTriangle **elements; /**< pointers to landscape
                				   triangles, one element is
                				   just a pointer, pointers are
                				   placed in array when new
                				   triangle is added to queue
                				   */
                LandscapeTriangle **Pool; /**< pool of free landscape
                			       triangles, this is used to avoid
                			       new/delete calls */
                LandscapeTriangle ***Arrays; /**< update arrays, used to store
                				  pointers to landscape
                				  triangles, one array of
                				  triangles is updated every
                				  frame */
                int *Sizes; /**< size of every update array */
                int PoolSize; /**< size of pool, i.e. how many triangles one can
                		   allocate */
                double maxx; /**< x size of landscape, used for priority
                	          calculation */
                double maxz; /**< z size of landscape, used for priority
                                  calculation */
                double ViewDistance; /**< distance of view, used for priority
                                          calculation */
                int NumberOfArrays; /**< how many update arrays exist */
                int SizeOfArrays; /**< how many triangles can be placed in one array */
                int CurrentArray; /**< array which is currently updated */
        public:
                SplitQueue(int _maxsize, double _maxx, double _maxz, double _vd);
                ~SplitQueue();

                void Add(LandscapeTriangle *element, Vector3D *CameraEye);
                /**< add landscape triangle to queue */
                void Remove(int pos);
                /**< remove landscape triangle at position pos from queue */
                int Empty();
                /**< is queue empty? */
                int GetSize();
                /**< returns size of queue */

                void Fix(int pos);
                /**< fixes tree, starts from position pos */

                void RecalcPriorities(Vector3D *CameraEye);
                /**< recalculates priorities of current update array */

                LandscapeTriangle *AllocateTriangle(BTTNode *_bttnode,TriangleInfoTree *_it,int _it_node);
                /**< allocates new landscape triangle */

                LandscapeTriangle *GetTriangle(BTTNode *bttnode);
                /**< returns landscape triangle for given bttnode */

                void RemoveBTTNodeFromQs(BTTNode *bttnode);
                /**< remove landscape triangle for given bttnode */

                LandscapeTriangle *GetHighest();
                /**< returns landscape triangle with highest priority */

                float GetHighestPriority();
                /**< returns highest priority */
};

#endif
