#include "triangleinfotree.h"
#include <SDL.h>
#include <cmath>
#include <stdlib.h>
#include <iostream>

TriangleInfoTree::TriangleInfoTree(double leftx, double leftz, double rightx, double rightz, double centerx, double centerz, int maxlevel, HeightMap *_hm,VertexStruct *_Vertices, double _x1, double _z1, IndicesTree *_indicestree)
{
        size=(int)pow(2.0,(double)maxlevel-1)-1;
        error=new float [size];
        node=1;
        hm=_hm;
        Calc(leftx,leftz,rightx,rightz,centerx,centerz);
        indicestree=_indicestree;
        px=(leftx+rightx+centerx)/3;
        py=0.0;
        pz=(leftz+rightz+centerz)/3;
        Visible=0;
}

TriangleInfoTree::~TriangleInfoTree()
{
        delete [] error;
}

double TriangleInfoTree::Calc(double leftx, double leftz, double rightx, double rightz, double centerx, double centerz)
{
        double v0;
        double LeftError,RightError;
        double v;

        if (node>size)
        {
                cout <<"TriangleInfoTree::Calc node>size\n";
                exit(-1);
        }

        v0=fabs(hm->GetHeight((leftx+rightx)/2,(leftz+rightz)/2)-(hm->GetHeight(leftx,leftz)+hm->GetHeight(rightx,rightz)/2));
        //v0=v0*0.01*sqrt((rightx-leftx)*(rightx-leftx)+(rightz-leftz)*(rightz-leftz));
        if (node>size/2)
        {
                v=v0;
        }
        else
        {
                node=node<<1;
                LeftError=Calc(centerx,centerz,leftx,leftz,(leftx+rightx)/2,(leftz+rightz)/2);
                node++;
                RightError=Calc(rightx,rightz,centerx,centerz,(leftx+rightx)/2,(leftz+rightz)/2);
                node=node>>1;
                if (LeftError>RightError)
                        v=LeftError;
                else
                        v=RightError;
                v=v+v0;
        }
        error[node-1]=v;
        return v;
}

int TriangleInfoTree::GetIndex(int p)
{
        return indicestree->GetIndex(p);
}
