#include <GL/gl.h>
#include "vector3d.h"
#include <iostream>
#include <cstdlib>
#include <sstream>
#include <cmath>

Vector3D::Vector3D ()
{
        x = 0;
        y = 0;
        z = 0;
}

Vector3D::Vector3D (double _x, double _y, double _z)
{
        x = _x;
        y = _y;
        z = _z;
}

Vector3D::Vector3D(const Vector3D &v)
{
        x = v.x;
        y = v.y;
        z = v.z;
}

Vector3D::Vector3D(const Vector3D &v1, const Vector3D &v2)
{
        x = v2.x - v1.x;
        y = v2.y - v1.y;
        z = v2.z - v1.z;
}

Vector3D::Vector3D(ifstream * f)
{
        (*f)>>x;
        (*f)>>y;
        (*f)>>z;
}

Vector3D &Vector3D::operator = (const Vector3D &v)
{
        x = v.x;
        y = v.y;
        z = v.z;
        return *this;
}

void Vector3D::operator += (const Vector3D &v)
{
        x += v.x;
        y += v.y;
        z += v.z;
}

Vector3D operator + (const Vector3D &v1, const Vector3D &v2)
{
        Vector3D v;
        v = v1;
        v += v2;
        return v;
}

void Vector3D::operator -= (const Vector3D &v)
{
        x -= v.x;
        y -= v.y;
        z -= v.z;
}

Vector3D operator - (const Vector3D &v1, const Vector3D &v2)
{
        Vector3D v;
        v = v1;
        v -= v2;
        return v;
}

Vector3D operator - (const Vector3D &v1)
{
        Vector3D v;
        v = v - v1;
        return v;
}

void Vector3D::operator *= (const double a)
{
        x *= a;
        y *= a;
        z *= a;
}

Vector3D operator * (const Vector3D &v1, double a)
{
        Vector3D v;
        v = Vector3D(v1);
        v *= a;
        return v;
}

void Vector3D::operator /= (const double a)
{
        x /= a;
        y /= a;
        z /= a;
}

Vector3D operator / (const Vector3D &v1, double a)
{
        Vector3D v;
        v = v1;
        v /= a;
        return v;
}

Vector3D operator / (const double a, Vector3D &v1)
{
        Vector3D v;
        v = Vector3D(a/v1.x,a/v1.y,a/v1.z);
        return v;
}

void Vector3D::Save (ofstream * f)
{
        (*f)<<x;
        (*f)<<y;
        (*f)<<z;
}

double Vector3D::Length ()
{
        return sqrt (x * x + y * y + z * z);
}

void Vector3D::Normalize ()
{
        double l;
        l = Length ();
        x = x / l;
        y = y / l;
        z = z / l;
}

void Vector3D::glTranslate ()
{
        glTranslatef (x, y, z);
}

void Vector3D::glScale ()
{
        glScalef (x, y, z);
}

void Vector3D::glRotate()
{
        glRotatef (z, 0.0, 0.0, 1.0);
        glRotatef (y, 0.0, 1.0, 0.0);
        glRotatef (x, 1.0, 0.0, 0.0);
}

void Vector3D::Print ()
{
        cout <<"x="<<x<<" y="<<y<<" z="<<z<<"\n";
}

void Vector3D::glNormal ()
{
        glNormal3f (x, y, z);
}

void Vector3D::glVertex ()
{
        glVertex3f (x, y, z);
}

double Vector3D::PointSideXZ (Vector3D * p, Vector3D * q)
{
        double det1, det2;
        det1 = p->x * q->z + q->x * z + x * p->z;
        det2 = q->z * x + z * p->x + p->z * q->x;
        return det1 - det2;
}

void Vector3D::ConvertRadToDeg ()
{
        x = 180 * x / M_PI;
        y = 180 * y / M_PI;
        z = 180 * z / M_PI;
}

double Vector3D::Distance (Vector3D *v)
{
        return sqrt ((x - v->x) * (x - v->x) + (y - v->y) * (y - v->y) + (z -
                        v->z) * (z - v->z));
}

Vector3D Vector3D::CrossProduct(Vector3D v1)
{
        Vector3D v;
        v.x = y * v1.z - z * v1.y;
        v.y = v1.x * z - x * v1.z;
        v.z = x * v1.y - y * v1.x;
        return v;
}

double Vector3D::DotProduct(Vector3D v)
{
        return x * v.x + y * v.y + z * v.z;
}

void Vector3D::SetMinMax (Vector3D * min, Vector3D * max)
{
        if (x < min->x)
                min->x = x;
        if (x > max->x)
                max->x = x;
        if (y < min->y)
                min->y = y;
        if (y > max->y)
                max->y = y;
        if (z < min->z)
                min->z = z;
        if (z > max->z)
                max->z = z;
}

Vector3D Vector3D::abs()
{
        Vector3D v;
        v.x=fabs(x);
        v.y=fabs(y);
        v.z=fabs(z);
        return v;
}

void Vector3D::WrapXZ(double maxx, double maxz)
{
        while (x<0.0)
                x+=maxx;
        while (x>maxx)
                x-=maxx;
        while (z<0.0)
                z+=maxz;
        while (z>maxz)
                z-=maxz;
}

string Vector3D::GetString()
{
        ostringstream oss;
        oss.str("");
        oss.setf(ios::fixed);
        oss.precision(2);
        oss<<x<<","<<y<<","<<z;
        return oss.str();
}
