#ifndef _VECTOR3D_H

#define _VECTOR3D_H 1

using namespace std;

#include <fstream>
#include <string>
#include <cmath>

#ifndef M_PI
# define M_PI           3.14159265358979323846
#endif

class Vector3D
                        /** Represents vector in 3D space. */
{
        private:
        public:
                double x, y, z;

                Vector3D ();	/**< default constructor */
                Vector3D (double _x, double _y, double _z); /**< pass all values */
                Vector3D (const Vector3D &v); /**< copy from other Vector3D */
                Vector3D (ifstream * f); /**< read from ifstream */
                Vector3D (const Vector3D &v1, const Vector3D &v2); /**< difference of two vectors */

                void Save (ofstream * f); /**< save to ofstream */

                Vector3D &operator = (const Vector3D &v); /**< returns copy of itself */
                void operator += (const Vector3D &v); /**< add v to itself */
                void operator -= (const Vector3D &v); /**< substract v from itself */
                void operator *= (double a); /**< multiply itself by a */
                void operator /= (double a); /**< divide itself by a */

                Vector3D CrossProduct(Vector3D v); /**< returns Cross Product of itself and v */
                Vector3D abs();	/**< returns Vector3D with signs set to + */
                double DotProduct(Vector3D v); /**< returns Dot Product of itself and v */

                void Normalize (); /**< normalize itself */
                double PointSideXZ (Vector3D * p, Vector3D * q); /**< calculate on which side
                							  of line pq it is located */
                double Length (); /**< returns length of Vector3D */
                void ConvertRadToDeg (); /**< convert angles x,y,z from Radians to Degrees */
                double Distance (Vector3D *v); /**< returns distance from itself to v */
                void SetMinMax (Vector3D * min, Vector3D * max); /**< compares ifself to two vectors,
                							  and changes them if any field
                			  is larger than in max or smaller 
                			  than in min*/

                void glScale (); /**< calls glScalef(x,y,z) */
                void glTranslate (); /**< calls glTranslatef(x,y,z) */
                void glRotate (); /**< calls glRotatef for z, y, x */
                void glNormal (); /**< calls glNormal3f(x,y,z) */
                void glVertex (); /**< calls glVertex3f(x,y,z) */

                void WrapXZ(double maxx, double maxz); /**< wrap x to (0..maxx) and z to (0..maxz)*/

                void Print(); /**< prints x,y,z to cout */
                string GetString(); /**< returns string with x,y,z */
};

Vector3D operator + (const Vector3D &v1, const Vector3D &v2);
Vector3D operator - (const Vector3D &v1, const Vector3D &v2);
Vector3D operator - (const Vector3D &v1);
Vector3D operator / (const double a, Vector3D &v1);
Vector3D operator * (const Vector3D &v1, double a);
Vector3D operator / (const Vector3D &v1, double a);

#endif
